### ms-frs1_frsrpc.idl
"""
RPC definitions for the following interfaces:
- frsrpc (v1.1): F5CC59B4-4264-101A-8C59-08002B2F8426
This file is auto-generated by midl-to-scapy, do not modify.
"""

import uuid


from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfStrLenField,
    NDRConfVarStrNullField,
    NDRConfVarStrNullFieldUtf16,
    NDRFullPointerField,
    NDRIntField,
    NDRPacketField,
    register_dcerpc_interface,
)


class PCOMM_PACKET(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Major", 0),
        NDRIntField("Minor", 0),
        NDRIntField("CsId", 0),
        NDRIntField("MemLen", 0),
        NDRIntField("PktLen", None, size_of="Pkt"),
        NDRIntField("UpkLen", 0),
        NDRFullPointerField(
            NDRConfStrLenField("Pkt", "", size_is=lambda pkt: pkt.PktLen), deferred=True
        ),
    ]


class FrsRpcSendCommPkt_Request(NDRPacket):
    fields_desc = [NDRPacketField("CommPkt", PCOMM_PACKET(), PCOMM_PACKET)]


class FrsRpcSendCommPkt_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class FrsRpcVerifyPromotionParent_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ParentAccount", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ParentPassword", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ReplicaSetName", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ReplicaSetType", "")),
        NDRIntField("PartnerAuthLevel", 0),
        NDRIntField("GuidSize", 0),
    ]


class FrsRpcVerifyPromotionParent_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class FrsRpcStartPromotionParent_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ParentAccount", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ParentPassword", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ReplicaSetName", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ReplicaSetType", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("CxtionName", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("PartnerName", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("PartnerPrincName", "")),
        NDRIntField("PartnerAuthLevel", 0),
        NDRIntField("GuidSize", None, size_of="ParentGuid"),
        NDRConfStrLenField("CxtionGuid", "", size_is=lambda pkt: pkt.GuidSize),
        NDRConfStrLenField("PartnerGuid", "", size_is=lambda pkt: pkt.GuidSize),
        NDRConfStrLenField("ParentGuid", "", size_is=lambda pkt: pkt.GuidSize),
    ]


class FrsRpcStartPromotionParent_Response(NDRPacket):
    fields_desc = [
        NDRConfStrLenField("ParentGuid", "", size_is=lambda pkt: pkt.GuidSize),
        NDRIntField("status", 0),
    ]


class FrsNOP_Request(NDRPacket):
    fields_desc = []


class FrsNOP_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


FRSRPC_OPNUMS = {
    0: DceRpcOp(FrsRpcSendCommPkt_Request, FrsRpcSendCommPkt_Response),
    1: DceRpcOp(
        FrsRpcVerifyPromotionParent_Request, FrsRpcVerifyPromotionParent_Response
    ),
    2: DceRpcOp(
        FrsRpcStartPromotionParent_Request, FrsRpcStartPromotionParent_Response
    ),
    3: DceRpcOp(FrsNOP_Request, FrsNOP_Response),
    # 4: Opnum4NotUsedOnWire,
    # 5: Opnum5NotUsedOnWire,
    # 6: Opnum6NotUsedOnWire,
    # 7: Opnum7NotUsedOnWire,
    # 8: Opnum8NotUsedOnWire,
    # 9: Opnum9NotUsedOnWire,
    # 10: Opnum10NotUsedOnWire
}
register_dcerpc_interface(
    name="frsrpc",
    uuid=uuid.UUID("F5CC59B4-4264-101A-8C59-08002B2F8426"),
    version="1.1",
    opnums=FRSRPC_OPNUMS,
)



### ms-raa.idl
"""
RPC definitions for the following interfaces:
- authzr (v0.0): 0b1c2170-5732-4e0e-8cd3-d9b16f3b84d7
This file is auto-generated by midl-to-scapy, do not modify.
"""

from enum import IntEnum
import uuid

from scapy.fields import StrFixedLenField
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRByteField,
    NDRConfFieldListField,
    NDRConfPacketListField,
    NDRConfStrLenField,
    NDRConfStrLenFieldUtf16,
    NDRContextHandle,
    NDRFullPointerField,
    NDRInt3264EnumField,
    NDRIntField,
    NDRLongField,
    NDRPacketField,
    NDRShortField,
    NDRSignedIntField,
    NDRSignedLongField,
    NDRUnionField,
    register_dcerpc_interface,
)


class AuthzrFreeContext_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("ContextHandle", NDRContextHandle(), NDRContextHandle)
    ]


class AuthzrFreeContext_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("ContextHandle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class RPC_SID_IDENTIFIER_AUTHORITY(NDRPacket):
    fields_desc = [StrFixedLenField("Value", "", length=6)]


class RPC_SID(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["SubAuthority"]
    fields_desc = [
        NDRByteField("Revision", 0),
        NDRByteField("SubAuthorityCount", None, size_of="SubAuthority"),
        NDRPacketField(
            "IdentifierAuthority",
            RPC_SID_IDENTIFIER_AUTHORITY(),
            RPC_SID_IDENTIFIER_AUTHORITY,
        ),
        NDRConfFieldListField(
            "SubAuthority",
            [],
            NDRIntField("", 0),
            size_is=lambda pkt: pkt.SubAuthorityCount,
            conformant_in_struct=True,
        ),
    ]


class LARGE_INTEGER(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [NDRSignedLongField("QuadPart", 0)]


class LUID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("LowPart", 0), NDRSignedIntField("HighPart", 0)]


class AuthzrInitializeContextFromSid_Request(NDRPacket):
    fields_desc = [
        NDRIntField("Flags", 0),
        NDRPacketField("Sid", RPC_SID(), RPC_SID),
        NDRFullPointerField(
            NDRPacketField("pExpirationTime", LARGE_INTEGER(), LARGE_INTEGER)
        ),
        NDRPacketField("Identifier", LUID(), LUID),
    ]


class AuthzrInitializeContextFromSid_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("ContextHandle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class AuthzrInitializeCompoundContext_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("UserContextHandle", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("DeviceContextHandle", NDRContextHandle(), NDRContextHandle),
    ]


class AuthzrInitializeCompoundContext_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("CompoundContextHandle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class GUID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Data1", 0),
        NDRShortField("Data2", 0),
        NDRShortField("Data3", 0),
        StrFixedLenField("Data4", "", length=8),
    ]


class OBJECT_TYPE_LIST(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRShortField("Level", 0),
        NDRIntField("Remaining", 0),
        NDRFullPointerField(NDRPacketField("ObjectType", GUID(), GUID), deferred=True),
    ]


class AUTHZR_ACCESS_REQUEST(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("DesiredAccess", 0),
        NDRFullPointerField(
            NDRPacketField("PrincipalSelfSid", RPC_SID(), RPC_SID), deferred=True
        ),
        NDRIntField("ObjectTypeListLength", None, size_of="ObjectTypeList"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "ObjectTypeList",
                [],
                OBJECT_TYPE_LIST,
                size_is=lambda pkt: pkt.ObjectTypeListLength,
            ),
            deferred=True,
        ),
    ]


class SR_SD(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("dwLength", None, size_of="pSrSd"),
        NDRFullPointerField(
            NDRConfStrLenField("pSrSd", "", size_is=lambda pkt: pkt.dwLength),
            deferred=True,
        ),
    ]


class AUTHZR_ACCESS_REPLY(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("ResultListLength", None, size_of="Error"),
        NDRFullPointerField(
            NDRConfFieldListField(
                "GrantedAccessMask",
                [],
                NDRIntField("GrantedAccessMask", 0),
                size_is=lambda pkt: pkt.ResultListLength,
            ),
            deferred=True,
        ),
        NDRFullPointerField(
            NDRConfFieldListField(
                "Error",
                [],
                NDRIntField("Error", 0),
                size_is=lambda pkt: pkt.ResultListLength,
            ),
            deferred=True,
        ),
    ]


class AuthzrAccessCheck_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("ContextHandle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("Flags", 0),
        NDRPacketField("pRequest", AUTHZR_ACCESS_REQUEST(), AUTHZR_ACCESS_REQUEST),
        NDRIntField("SecurityDescriptorCount", None, size_of="pSecurityDescriptors"),
        NDRConfPacketListField(
            "pSecurityDescriptors",
            [],
            SR_SD,
            size_is=lambda pkt: pkt.SecurityDescriptorCount,
        ),
        NDRPacketField("pReply", AUTHZR_ACCESS_REPLY(), AUTHZR_ACCESS_REPLY),
    ]


class AuthzrAccessCheck_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pReply", AUTHZR_ACCESS_REPLY(), AUTHZR_ACCESS_REPLY),
        NDRIntField("status", 0),
    ]


class AUTHZ_CONTEXT_INFORMATION_CLASS(IntEnum):
    AuthzContextInfoUserSid = 1
    AuthzContextInfoGroupsSids = 2
    AuthzContextInfoRestrictedSids = 3
    ReservedEnumValue4 = 4
    ReservedEnumValue5 = 5
    ReservedEnumValue6 = 6
    ReservedEnumValue7 = 7
    ReservedEnumValue8 = 8
    ReservedEnumValue9 = 9
    ReservedEnumValue10 = 10
    ReservedEnumValue11 = 11
    AuthzContextInfoDeviceSids = 12
    AuthzContextInfoUserClaims = 13
    AuthzContextInfoDeviceClaims = 14
    ReservedEnumValue15 = 15
    ReservedEnumValue16 = 16


class AUTHZR_SID_AND_ATTRIBUTES(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRFullPointerField(NDRPacketField("Sid", RPC_SID(), RPC_SID), deferred=True),
        NDRIntField("Attributes", 0),
    ]


class AUTHZR_TOKEN_USER(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRPacketField("User", AUTHZR_SID_AND_ATTRIBUTES(), AUTHZR_SID_AND_ATTRIBUTES)
    ]


class AUTHZR_TOKEN_GROUPS(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["Groups"]
    fields_desc = [
        NDRIntField("GroupCount", None, size_of="Groups"),
        NDRConfPacketListField(
            "Groups",
            [],
            AUTHZR_SID_AND_ATTRIBUTES,
            size_is=lambda pkt: pkt.GroupCount,
            conformant_in_struct=True,
        ),
    ]


class AUTHZR_SECURITY_ATTRIBUTE_STRING_VALUE(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Length", None, size_of="Value"),
        NDRFullPointerField(
            NDRConfStrLenFieldUtf16("Value", "", size_is=lambda pkt: pkt.Length),
            deferred=True,
        ),
    ]


class AUTHZR_SECURITY_ATTRIBUTE_V1_VALUE(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRShortField("ValueType", 0),
        NDRUnionField(
            [
                (
                    NDRSignedLongField("AttributeUnion", 0),
                    (
                        (lambda pkt: getattr(pkt, "ValueType", None) == 1),
                        (lambda _, val: val.tag == 1),
                    ),
                ),
                (
                    NDRLongField("AttributeUnion", 0),
                    (
                        (lambda pkt: getattr(pkt, "ValueType", None) in [2, 6]),
                        (lambda _, val: val.tag in [2, 6]),
                    ),
                ),
                (
                    NDRPacketField(
                        "AttributeUnion",
                        AUTHZR_SECURITY_ATTRIBUTE_STRING_VALUE(),
                        AUTHZR_SECURITY_ATTRIBUTE_STRING_VALUE,
                    ),
                    (
                        (lambda pkt: getattr(pkt, "ValueType", None) == 3),
                        (lambda _, val: val.tag == 3),
                    ),
                ),
            ],
            StrFixedLenField("AttributeUnion", "", length=0),
            align=(2, 8),
            switch_fmt=("H", "H"),
        ),
    ]


class AUTHZR_SECURITY_ATTRIBUTE_V1(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Length", None, size_of="Value"),
        NDRFullPointerField(
            NDRConfStrLenFieldUtf16("Value", "", size_is=lambda pkt: pkt.Length),
            deferred=True,
        ),
        NDRShortField("ValueType", 0),
        NDRShortField("Reserved", 0),
        NDRIntField("Flags", 0),
        NDRIntField("ValueCount", None, size_of="Values"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "Values",
                [],
                AUTHZR_SECURITY_ATTRIBUTE_V1_VALUE,
                size_is=lambda pkt: pkt.ValueCount,
            ),
            deferred=True,
        ),
    ]


class AUTHZR_SECURITY_ATTRIBUTES_INFORMATION(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRShortField("Version", 0),
        NDRShortField("Reserved", 0),
        NDRIntField("AttributeCount", None, size_of="Attributes"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "Attributes",
                [],
                AUTHZR_SECURITY_ATTRIBUTE_V1,
                size_is=lambda pkt: pkt.AttributeCount,
            ),
            deferred=True,
        ),
    ]


class AUTHZR_CONTEXT_INFORMATION(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRShortField("ValueType", 0),
        NDRUnionField(
            [
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "ContextInfoUnion", AUTHZR_TOKEN_USER(), AUTHZR_TOKEN_USER
                        ),
                        deferred=True,
                    ),
                    (
                        (lambda pkt: getattr(pkt, "ValueType", None) == 1),
                        (lambda _, val: val.tag == 1),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "ContextInfoUnion",
                            AUTHZR_TOKEN_GROUPS(),
                            AUTHZR_TOKEN_GROUPS,
                        ),
                        deferred=True,
                    ),
                    (
                        (lambda pkt: getattr(pkt, "ValueType", None) in [2, 3, 12]),
                        (lambda _, val: val.tag in [2, 3, 12]),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "ContextInfoUnion",
                            AUTHZR_SECURITY_ATTRIBUTES_INFORMATION(),
                            AUTHZR_SECURITY_ATTRIBUTES_INFORMATION,
                        ),
                        deferred=True,
                    ),
                    (
                        (lambda pkt: getattr(pkt, "ValueType", None) in [13, 14]),
                        (lambda _, val: val.tag in [13, 14]),
                    ),
                ),
            ],
            StrFixedLenField("ContextInfoUnion", "", length=0),
            align=(2, 8),
            switch_fmt=("H", "H"),
        ),
    ]


class AuthzGetInformationFromContext_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("ContextHandle", NDRContextHandle(), NDRContextHandle),
        NDRInt3264EnumField("InfoClass", 0, AUTHZ_CONTEXT_INFORMATION_CLASS),
    ]


class AuthzGetInformationFromContext_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "ppContextInformation",
                AUTHZR_CONTEXT_INFORMATION(),
                AUTHZR_CONTEXT_INFORMATION,
            )
        ),
        NDRIntField("status", 0),
    ]


class AUTHZ_SECURITY_ATTRIBUTE_OPERATION(IntEnum):
    AUTHZ_SECURITY_ATTRIBUTE_OPERATION_NONE = 0
    AUTHZ_SECURITY_ATTRIBUTE_OPERATION_REPLACE_ALL = 1
    AUTHZ_SECURITY_ATTRIBUTE_OPERATION_ADD = 2
    AUTHZ_SECURITY_ATTRIBUTE_OPERATION_DELETE = 3
    AUTHZ_SECURITY_ATTRIBUTE_OPERATION_REPLACE = 4


class AuthzrModifyClaims_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("ContextHandle", NDRContextHandle(), NDRContextHandle),
        NDRInt3264EnumField("ClaimClass", 0, AUTHZ_CONTEXT_INFORMATION_CLASS),
        NDRIntField("OperationCount", None, size_of="pClaimOperations"),
        NDRConfFieldListField(
            "pClaimOperations",
            [],
            NDRInt3264EnumField(
                "pClaimOperations", 0, AUTHZ_SECURITY_ATTRIBUTE_OPERATION
            ),
            size_is=lambda pkt: pkt.OperationCount,
        ),
        NDRFullPointerField(
            NDRPacketField(
                "pClaims",
                AUTHZR_SECURITY_ATTRIBUTES_INFORMATION(),
                AUTHZR_SECURITY_ATTRIBUTES_INFORMATION,
            )
        ),
    ]


class AuthzrModifyClaims_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class AUTHZ_SID_OPERATION(IntEnum):
    AUTHZ_SID_OPERATION_NONE = 0
    AUTHZ_SID_OPERATION_REPLACE_ALL = 1
    AUTHZ_SID_OPERATION_ADD = 2
    AUTHZ_SID_OPERATION_DELETE = 3
    AUTHZ_SID_OPERATION_REPLACE = 4


class AuthzrModifySids_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("ContextHandle", NDRContextHandle(), NDRContextHandle),
        NDRInt3264EnumField("SidClass", 0, AUTHZ_CONTEXT_INFORMATION_CLASS),
        NDRIntField("OperationCount", None, size_of="pSidOperations"),
        NDRConfFieldListField(
            "pSidOperations",
            [],
            NDRInt3264EnumField("pSidOperations", 0, AUTHZ_SID_OPERATION),
            size_is=lambda pkt: pkt.OperationCount,
        ),
        NDRFullPointerField(
            NDRPacketField("pSids", AUTHZR_TOKEN_GROUPS(), AUTHZR_TOKEN_GROUPS)
        ),
    ]


class AuthzrModifySids_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


AUTHZR_OPNUMS = {
    0: DceRpcOp(AuthzrFreeContext_Request, AuthzrFreeContext_Response),
    1: DceRpcOp(
        AuthzrInitializeContextFromSid_Request, AuthzrInitializeContextFromSid_Response
    ),
    2: DceRpcOp(
        AuthzrInitializeCompoundContext_Request,
        AuthzrInitializeCompoundContext_Response,
    ),
    3: DceRpcOp(AuthzrAccessCheck_Request, AuthzrAccessCheck_Response),
    4: DceRpcOp(
        AuthzGetInformationFromContext_Request, AuthzGetInformationFromContext_Response
    ),
    5: DceRpcOp(AuthzrModifyClaims_Request, AuthzrModifyClaims_Response),
    6: DceRpcOp(AuthzrModifySids_Request, AuthzrModifySids_Response),
}
register_dcerpc_interface(
    name="authzr",
    uuid=uuid.UUID("0b1c2170-5732-4e0e-8cd3-d9b16f3b84d7"),
    version="0.0",
    opnums=AUTHZR_OPNUMS,
)



### ms-ssp.idl
"""
RPC definitions for the following interfaces:
- ISingleSignonRemoteMasterSecret (v1.0): 9D07CA0D-8F02-4ed5-B727-ACF37FEA5BBC
This file is auto-generated by midl-to-scapy, do not modify.
"""

import uuid


from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfStrLenField,
    NDRIntField,
    register_dcerpc_interface,
)


class RemoteGetMasterSecret_Request(NDRPacket):
    fields_desc = [NDRIntField("pcbSecret", 0)]


class RemoteGetMasterSecret_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pcbSecret", None, size_of="pbSecret"),
        NDRConfStrLenField("pbSecret", "", size_is=lambda pkt: pkt.pcbSecret),
        NDRIntField("status", 0),
    ]


ISINGLESIGNONREMOTEMASTERSECRET_OPNUMS = {
    0: DceRpcOp(RemoteGetMasterSecret_Request, RemoteGetMasterSecret_Response)
}
register_dcerpc_interface(
    name="ISingleSignonRemoteMasterSecret",
    uuid=uuid.UUID("9D07CA0D-8F02-4ed5-B727-ACF37FEA5BBC"),
    version="1.0",
    opnums=ISINGLESIGNONREMOTEMASTERSECRET_OPNUMS,
)



### ms-frs2.idl
"""
RPC definitions for the following interfaces:
- FrsTransport (v1.0): 897e2e5f-93f3-4376-9c9c-fd2277495c27
This file is auto-generated by midl-to-scapy, do not modify.
"""

from enum import IntEnum
import uuid

from scapy.fields import PacketListField, StrFixedLenField
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfPacketListField,
    NDRConfStrLenField,
    NDRConfVarPacketListField,
    NDRConfVarStrLenField,
    NDRContextHandle,
    NDRFullPointerField,
    NDRInt3264EnumField,
    NDRIntField,
    NDRLongField,
    NDRPacketField,
    NDRShortField,
    NDRSignedByteField,
    NDRSignedIntField,
    NDRUnionField,
    NDRVarStrLenField,
    NDRVarStrLenFieldUtf16,
    register_dcerpc_interface,
)


class GUID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Data1", 0),
        NDRShortField("Data2", 0),
        NDRShortField("Data3", 0),
        StrFixedLenField("Data4", "", length=8),
    ]


class CheckConnectivity_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("replicaSetId", GUID(), GUID),
        NDRPacketField("connectionId", GUID(), GUID),
    ]


class CheckConnectivity_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class EstablishConnection_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("replicaSetId", GUID(), GUID),
        NDRPacketField("connectionId", GUID(), GUID),
        NDRIntField("downstreamProtocolVersion", 0),
        NDRIntField("downstreamFlags", 0),
    ]


class EstablishConnection_Response(NDRPacket):
    fields_desc = [
        NDRIntField("upstreamProtocolVersion", 0),
        NDRIntField("upstreamFlags", 0),
        NDRIntField("status", 0),
    ]


class EstablishSession_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("connectionId", GUID(), GUID),
        NDRPacketField("contentSetId", GUID(), GUID),
    ]


class EstablishSession_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class UPDATE_REQUEST_TYPE(IntEnum):
    UPDATE_REQUEST_ALL = 0
    UPDATE_REQUEST_TOMBSTONES = 1
    UPDATE_REQUEST_LIVE = 2


class FRS_VERSION_VECTOR(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRPacketField("dbGuid", GUID(), GUID),
        NDRLongField("low", 0),
        NDRLongField("high", 0),
    ]


class FILETIME(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("dwLowDateTime", 0), NDRIntField("dwHighDateTime", 0)]


class FRS_UPDATE(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRSignedIntField("present", 0),
        NDRSignedIntField("nameConflict", 0),
        NDRIntField("attributes", 0),
        NDRPacketField("fence", FILETIME(), FILETIME),
        NDRPacketField("clock", FILETIME(), FILETIME),
        NDRPacketField("createTime", FILETIME(), FILETIME),
        NDRPacketField("contentSetId", GUID(), GUID),
        StrFixedLenField("hash", "", length=20),
        StrFixedLenField("rdcSimilarity", "", length=16),
        NDRPacketField("uidDbGuid", GUID(), GUID),
        NDRLongField("uidVersion", 0),
        NDRPacketField("gvsnDbGuid", GUID(), GUID),
        NDRLongField("gvsnVersion", 0),
        NDRPacketField("parentDbGuid", GUID(), GUID),
        NDRLongField("parentVersion", 0),
        NDRVarStrLenFieldUtf16("name", ""),
        NDRSignedIntField("flags", 0),
    ]


class UPDATE_STATUS(IntEnum):
    UPDATE_STATUS_DONE = 2
    UPDATE_STATUS_MORE = 3


class RequestUpdates_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("connectionId", GUID(), GUID),
        NDRPacketField("contentSetId", GUID(), GUID),
        NDRIntField("creditsAvailable", 0),
        NDRSignedIntField("hashRequested", 0),
        NDRInt3264EnumField("updateRequestType", 0, UPDATE_REQUEST_TYPE),
        NDRIntField("versionVectorDiffCount", None, size_of="versionVectorDiff"),
        NDRConfPacketListField(
            "versionVectorDiff",
            [],
            FRS_VERSION_VECTOR,
            size_is=lambda pkt: pkt.versionVectorDiffCount,
        ),
    ]


class RequestUpdates_Response(NDRPacket):
    fields_desc = [
        NDRConfVarPacketListField(
            "frsUpdate",
            [],
            FRS_UPDATE,
            size_is=lambda pkt: pkt.creditsAvailable,
            length_is=lambda pkt: pkt.updateCount,
        ),
        NDRIntField("updateCount", None, size_of="frsUpdate"),
        NDRInt3264EnumField("updateStatus", 0, UPDATE_STATUS),
        NDRPacketField("gvsnDbGuid", GUID(), GUID),
        NDRLongField("gvsnVersion", 0),
        NDRIntField("status", 0),
    ]


class VERSION_REQUEST_TYPE(IntEnum):
    REQUEST_NORMAL_SYNC = 0
    REQUEST_SLOW_SYNC = 1
    REQUEST_SUBORDINATE_SYNC = 2


class VERSION_CHANGE_TYPE(IntEnum):
    CHANGE_NOTIFY = 0
    CHANGE_ALL = 2


class RequestVersionVector_Request(NDRPacket):
    fields_desc = [
        NDRIntField("sequenceNumber", 0),
        NDRPacketField("connectionId", GUID(), GUID),
        NDRPacketField("contentSetId", GUID(), GUID),
        NDRInt3264EnumField("requestType", 0, VERSION_REQUEST_TYPE),
        NDRInt3264EnumField("changeType", 0, VERSION_CHANGE_TYPE),
        NDRLongField("vvGeneration", 0),
    ]


class RequestVersionVector_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class SYSTEMTIME(NDRPacket):
    ALIGNMENT = (2, 2)
    fields_desc = [
        NDRShortField("wYear", 0),
        NDRShortField("wMonth", 0),
        NDRShortField("wDayOfWeek", 0),
        NDRShortField("wDay", 0),
        NDRShortField("wHour", 0),
        NDRShortField("wMinute", 0),
        NDRShortField("wSecond", 0),
        NDRShortField("wMilliseconds", 0),
    ]


class FRS_EPOQUE_VECTOR(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRPacketField("machine", GUID(), GUID),
        NDRPacketField("epoque", SYSTEMTIME(), SYSTEMTIME),
    ]


class FRS_ASYNC_VERSION_VECTOR_RESPONSE(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRLongField("vvGeneration", 0),
        NDRIntField("versionVectorCount", None, size_of="versionVector"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "versionVector",
                [],
                FRS_VERSION_VECTOR,
                size_is=lambda pkt: pkt.versionVectorCount,
            ),
            deferred=True,
        ),
        NDRIntField("epoqueVectorCount", None, size_of="epoqueVector"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "epoqueVector",
                [],
                FRS_EPOQUE_VECTOR,
                size_is=lambda pkt: pkt.epoqueVectorCount,
            ),
            deferred=True,
        ),
    ]


class FRS_ASYNC_RESPONSE_CONTEXT(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRIntField("sequenceNumber", 0),
        NDRIntField("status", 0),
        NDRPacketField(
            "result",
            FRS_ASYNC_VERSION_VECTOR_RESPONSE(),
            FRS_ASYNC_VERSION_VECTOR_RESPONSE,
        ),
    ]


class AsyncPoll_Request(NDRPacket):
    fields_desc = [NDRPacketField("connectionId", GUID(), GUID)]


class AsyncPoll_Response(NDRPacket):
    fields_desc = [
        NDRPacketField(
            "response", FRS_ASYNC_RESPONSE_CONTEXT(), FRS_ASYNC_RESPONSE_CONTEXT
        ),
        NDRIntField("status", 0),
    ]


class RECORDS_STATUS(IntEnum):
    RECORDS_STATUS_DONE = 0
    RECORDS_STATUS_MORE = 1


class RequestRecords_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("connectionId", GUID(), GUID),
        NDRPacketField("contentSetId", GUID(), GUID),
        NDRPacketField("uidDbGuid", GUID(), GUID),
        NDRLongField("uidVersion", 0),
        NDRIntField("maxRecords", 0),
    ]


class RequestRecords_Response(NDRPacket):
    fields_desc = [
        NDRIntField("maxRecords", 0),
        NDRIntField("numRecords", 0),
        NDRIntField("numBytes", None, size_of="compressedRecords"),
        NDRConfStrLenField("compressedRecords", "", size_is=lambda pkt: pkt.numBytes),
        NDRInt3264EnumField("recordsStatus", 0, RECORDS_STATUS),
        NDRIntField("status", 0),
    ]


class FRS_UPDATE_CANCEL_DATA(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRPacketField("blockingUpdate", FRS_UPDATE(), FRS_UPDATE),
        NDRPacketField("contentSetId", GUID(), GUID),
        NDRPacketField("gvsnDatabaseId", GUID(), GUID),
        NDRPacketField("uidDatabaseId", GUID(), GUID),
        NDRPacketField("parentDatabaseId", GUID(), GUID),
        NDRLongField("gvsnVersion", 0),
        NDRLongField("uidVersion", 0),
        NDRLongField("parentVersion", 0),
        NDRIntField("cancelType", 0),
        NDRSignedIntField("isUidValid", 0),
        NDRSignedIntField("isParentUidValid", 0),
        NDRSignedIntField("isBlockerValid", 0),
    ]


class UpdateCancel_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("connectionId", GUID(), GUID),
        NDRPacketField("cancelData", FRS_UPDATE_CANCEL_DATA(), FRS_UPDATE_CANCEL_DATA),
    ]


class UpdateCancel_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class RawGetFileData_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("serverContext", NDRContextHandle(), NDRContextHandle),
        NDRIntField("bufferSize", 0),
    ]


class RawGetFileData_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("serverContext", NDRContextHandle(), NDRContextHandle),
        NDRConfVarStrLenField(
            "dataBuffer",
            "",
            size_is=lambda pkt: pkt.bufferSize,
            length_is=lambda pkt: pkt.sizeRead,
        ),
        NDRIntField("sizeRead", None, size_of="dataBuffer"),
        NDRSignedIntField("isEndOfFile", 0),
        NDRIntField("status", 0),
    ]


class RdcGetSignatures_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("serverContext", NDRContextHandle(), NDRContextHandle),
        NDRSignedByteField("level", 0),
        NDRLongField("offset", 0),
        NDRIntField("length", 0),
    ]


class RdcGetSignatures_Response(NDRPacket):
    fields_desc = [
        NDRConfVarStrLenField(
            "buffer",
            "",
            size_is=lambda pkt: pkt.length,
            length_is=lambda pkt: pkt.sizeRead,
        ),
        NDRIntField("sizeRead", None, size_of="buffer"),
        NDRIntField("status", 0),
    ]


class FRS_RDC_SOURCE_NEED(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [NDRLongField("needOffset", 0), NDRLongField("needSize", 0)]


class RdcPushSourceNeeds_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("serverContext", NDRContextHandle(), NDRContextHandle),
        NDRConfPacketListField(
            "sourceNeeds", [], FRS_RDC_SOURCE_NEED, size_is=lambda pkt: pkt.needCount
        ),
        NDRIntField("needCount", None, size_of="sourceNeeds"),
    ]


class RdcPushSourceNeeds_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class RdcGetFileData_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("serverContext", NDRContextHandle(), NDRContextHandle),
        NDRIntField("bufferSize", 0),
    ]


class RdcGetFileData_Response(NDRPacket):
    fields_desc = [
        NDRConfVarStrLenField(
            "dataBuffer",
            "",
            size_is=lambda pkt: pkt.bufferSize,
            length_is=lambda pkt: pkt.sizeReturned,
        ),
        NDRIntField("sizeReturned", None, size_of="dataBuffer"),
        NDRIntField("status", 0),
    ]


class RdcClose_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("serverContext", NDRContextHandle(), NDRContextHandle)
    ]


class RdcClose_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("serverContext", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class FRS_REQUESTED_STAGING_POLICY(IntEnum):
    SERVER_DEFAULT = 0
    STAGING_REQUIRED = 1
    RESTAGING_REQUIRED = 2


class RDC_FILE_COMPRESSION_TYPES(IntEnum):
    RDC_UNCOMPRESSED = 0
    RDC_XPRESS = 1


class RDC_CHUNKER_ALGORITHM(IntEnum):
    RDC_FILTERGENERIC = 0
    RDC_FILTERMAX = 1
    RDC_FILTERPOINT = 2
    RDC_MAXALGORITHM = 3


class FRS_RDC_PARAMETERS_GENERIC(NDRPacket):
    ALIGNMENT = (2, 2)
    fields_desc = [
        NDRShortField("chunkerType", 0),
        StrFixedLenField("chunkerParameters", "", length=64),
    ]


class FRS_RDC_PARAMETERS_FILTERMAX(NDRPacket):
    ALIGNMENT = (2, 2)
    fields_desc = [NDRShortField("horizonSize", 0), NDRShortField("windowSize", 0)]


class FRS_RDC_PARAMETERS_FILTERPOINT(NDRPacket):
    ALIGNMENT = (2, 2)
    fields_desc = [NDRShortField("minChunkSize", 0), NDRShortField("maxChunkSize", 0)]


class FRS_RDC_PARAMETERS(NDRPacket):
    ALIGNMENT = (2, 4)
    fields_desc = [
        NDRShortField("rdcChunkerAlgorithm", 0),
        NDRUnionField(
            [
                (
                    NDRPacketField(
                        "u", FRS_RDC_PARAMETERS_GENERIC(), FRS_RDC_PARAMETERS_GENERIC
                    ),
                    (
                        (
                            lambda pkt: getattr(pkt, "rdcChunkerAlgorithm", None)
                            == RDC_CHUNKER_ALGORITHM.RDC_FILTERGENERIC
                        ),
                        (
                            lambda _, val: val.tag
                            == RDC_CHUNKER_ALGORITHM.RDC_FILTERGENERIC
                        ),
                    ),
                ),
                (
                    NDRPacketField(
                        "u",
                        FRS_RDC_PARAMETERS_FILTERMAX(),
                        FRS_RDC_PARAMETERS_FILTERMAX,
                    ),
                    (
                        (
                            lambda pkt: getattr(pkt, "rdcChunkerAlgorithm", None)
                            == RDC_CHUNKER_ALGORITHM.RDC_FILTERMAX
                        ),
                        (lambda _, val: val.tag == RDC_CHUNKER_ALGORITHM.RDC_FILTERMAX),
                    ),
                ),
                (
                    NDRPacketField(
                        "u",
                        FRS_RDC_PARAMETERS_FILTERPOINT(),
                        FRS_RDC_PARAMETERS_FILTERPOINT,
                    ),
                    (
                        (
                            lambda pkt: getattr(pkt, "rdcChunkerAlgorithm", None)
                            == RDC_CHUNKER_ALGORITHM.RDC_FILTERPOINT
                        ),
                        (
                            lambda _, val: val.tag
                            == RDC_CHUNKER_ALGORITHM.RDC_FILTERPOINT
                        ),
                    ),
                ),
            ],
            StrFixedLenField("u", "", length=0),
            align=(2, 4),
            switch_fmt=("H", "H"),
        ),
    ]


class FRS_RDC_FILEINFO(NDRPacket):
    ALIGNMENT = (8, 8)
    DEPORTED_CONFORMANTS = ["rdcFilterParameters"]
    fields_desc = [
        NDRLongField("onDiskFileSize", 0),
        NDRLongField("fileSizeEstimate", 0),
        NDRShortField("rdcVersion", 0),
        NDRShortField("rdcMinimumCompatibleVersion", 0),
        NDRSignedByteField("rdcSignatureLevels", None, size_of="rdcFilterParameters"),
        NDRInt3264EnumField("compressionAlgorithm", 0, RDC_FILE_COMPRESSION_TYPES),
        NDRConfPacketListField(
            "rdcFilterParameters",
            [],
            FRS_RDC_PARAMETERS,
            size_is=lambda pkt: pkt.rdcSignatureLevels,
            conformant_in_struct=True,
        ),
    ]


class InitializeFileTransferAsync_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("connectionId", GUID(), GUID),
        NDRPacketField("frsUpdate", FRS_UPDATE(), FRS_UPDATE),
        NDRSignedIntField("rdcDesired", 0),
        NDRInt3264EnumField("stagingPolicy", 0, FRS_REQUESTED_STAGING_POLICY),
        NDRIntField("bufferSize", 0),
    ]


class InitializeFileTransferAsync_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("frsUpdate", FRS_UPDATE(), FRS_UPDATE),
        NDRInt3264EnumField("stagingPolicy", 0, FRS_REQUESTED_STAGING_POLICY),
        NDRPacketField("serverContext", NDRContextHandle(), NDRContextHandle),
        NDRFullPointerField(
            NDRPacketField("rdcFileInfo", FRS_RDC_FILEINFO(), FRS_RDC_FILEINFO)
        ),
        NDRConfVarStrLenField(
            "dataBuffer",
            "",
            size_is=lambda pkt: pkt.bufferSize,
            length_is=lambda pkt: pkt.sizeRead,
        ),
        NDRIntField("sizeRead", None, size_of="dataBuffer"),
        NDRSignedIntField("isEndOfFile", 0),
        NDRIntField("status", 0),
    ]


class RawGetFileDataAsync_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("serverContext", NDRContextHandle(), NDRContextHandle)
    ]


class RawGetFileDataAsync_Response(NDRPacket):
    fields_desc = [NDRSignedByteField("bytePipe", 0), NDRIntField("status", 0)]


class RdcGetFileDataAsync_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("serverContext", NDRContextHandle(), NDRContextHandle)
    ]


class RdcGetFileDataAsync_Response(NDRPacket):
    fields_desc = [NDRSignedByteField("bytePipe", 0), NDRIntField("status", 0)]


class RdcFileDataTransferKeepAlive_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("serverContext", NDRContextHandle(), NDRContextHandle)
    ]


class RdcFileDataTransferKeepAlive_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


FRSTRANSPORT_OPNUMS = {
    0: DceRpcOp(CheckConnectivity_Request, CheckConnectivity_Response),
    1: DceRpcOp(EstablishConnection_Request, EstablishConnection_Response),
    2: DceRpcOp(EstablishSession_Request, EstablishSession_Response),
    3: DceRpcOp(RequestUpdates_Request, RequestUpdates_Response),
    4: DceRpcOp(RequestVersionVector_Request, RequestVersionVector_Response),
    5: DceRpcOp(AsyncPoll_Request, AsyncPoll_Response),
    6: DceRpcOp(RequestRecords_Request, RequestRecords_Response),
    7: DceRpcOp(UpdateCancel_Request, UpdateCancel_Response),
    8: DceRpcOp(RawGetFileData_Request, RawGetFileData_Response),
    9: DceRpcOp(RdcGetSignatures_Request, RdcGetSignatures_Response),
    10: DceRpcOp(RdcPushSourceNeeds_Request, RdcPushSourceNeeds_Response),
    11: DceRpcOp(RdcGetFileData_Request, RdcGetFileData_Response),
    12: DceRpcOp(RdcClose_Request, RdcClose_Response),
    13: DceRpcOp(
        InitializeFileTransferAsync_Request, InitializeFileTransferAsync_Response
    ),
    # 14: Opnum14NotUsedOnWire,
    15: DceRpcOp(RawGetFileDataAsync_Request, RawGetFileDataAsync_Response),
    16: DceRpcOp(RdcGetFileDataAsync_Request, RdcGetFileDataAsync_Response),
    17: DceRpcOp(
        RdcFileDataTransferKeepAlive_Request, RdcFileDataTransferKeepAlive_Response
    ),
}
register_dcerpc_interface(
    name="FrsTransport",
    uuid=uuid.UUID("897e2e5f-93f3-4376-9c9c-fd2277495c27"),
    version="1.0",
    opnums=FRSTRANSPORT_OPNUMS,
)



### ms-cmpo.idl
"""
RPC definitions for the following interfaces:
- IXnRemote (v1.0): 906B0CE0-C70B-1067-B317-00DD010662DA
This file is auto-generated by midl-to-scapy, do not modify.
"""

from enum import IntEnum
import uuid


from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfStrLenField,
    NDRConfVarStrLenField,
    NDRConfVarStrLenFieldUtf16,
    NDRContextHandle,
    NDRInt3264EnumField,
    NDRIntField,
    NDRPacketField,
    register_dcerpc_interface,
)


class SESSION_RANK(IntEnum):
    SRANK_PRIMARY = 1
    SRANK_SECONDARY = 2


class Poke_Request(NDRPacket):
    fields_desc = [
        NDRInt3264EnumField("sRank", 0, SESSION_RANK),
        NDRConfVarStrLenField("pszCalleeUuid", "", size_is=lambda pkt: pkt.GUID_LENGTH),
        NDRConfVarStrLenField(
            "pszHostName", "", size_is=lambda pkt: (pkt.MAX_COMPUTERNAME_LENGTH + 1)
        ),
        NDRConfVarStrLenField("pszUuidString", "", size_is=lambda pkt: pkt.GUID_LENGTH),
        NDRIntField("dwcbSizeOfBlob", None, size_of="rguchBlob"),
        NDRConfStrLenField("rguchBlob", "", size_is=lambda pkt: pkt.dwcbSizeOfBlob),
    ]


class Poke_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class BIND_VERSION_SET(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("dwMinLevelOne", 0),
        NDRIntField("dwMaxLevelOne", 0),
        NDRIntField("dwMinLevelTwo", 0),
        NDRIntField("dwMaxLevelTwo", 0),
        NDRIntField("dwMinLevelThree", 0),
        NDRIntField("dwMaxLevelThree", 0),
    ]


class BOUND_VERSION_SET(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("dwLevelOneAccepted", 0),
        NDRIntField("dwLevelTwoAccepted", 0),
        NDRIntField("dwLevelThreeAccepted", 0),
    ]


class BuildContext_Request(NDRPacket):
    fields_desc = [
        NDRInt3264EnumField("sRank", 0, SESSION_RANK),
        NDRPacketField("BindVersionSet", BIND_VERSION_SET(), BIND_VERSION_SET),
        NDRConfVarStrLenField("pszCalleeUuid", "", size_is=lambda pkt: pkt.GUID_LENGTH),
        NDRConfVarStrLenField(
            "pszHostName", "", size_is=lambda pkt: (pkt.MAX_COMPUTERNAME_LENGTH + 1)
        ),
        NDRConfVarStrLenField("pszUuidString", "", size_is=lambda pkt: pkt.GUID_LENGTH),
        NDRConfVarStrLenField("pszGuidIn", "", size_is=lambda pkt: pkt.GUID_LENGTH),
        NDRConfVarStrLenField("pszGuidOut", "", size_is=lambda pkt: pkt.GUID_LENGTH),
        NDRPacketField("pBoundVersionSet", BOUND_VERSION_SET(), BOUND_VERSION_SET),
        NDRIntField("dwcbSizeOfBlob", None, size_of="rguchBlob"),
        NDRConfStrLenField("rguchBlob", "", size_is=lambda pkt: pkt.dwcbSizeOfBlob),
    ]


class BuildContext_Response(NDRPacket):
    fields_desc = [
        NDRConfVarStrLenField("pszGuidOut", "", size_is=lambda pkt: pkt.GUID_LENGTH),
        NDRPacketField("pBoundVersionSet", BOUND_VERSION_SET(), BOUND_VERSION_SET),
        NDRPacketField("ppHandle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class RESOURCE_TYPE(IntEnum):
    RT_CONNECTIONS = 0


class NegotiateResources_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("phContext", NDRContextHandle(), NDRContextHandle),
        NDRInt3264EnumField("resourceType", 0, RESOURCE_TYPE),
        NDRIntField("dwcRequested", 0),
        NDRIntField("pdwcAccepted", 0),
    ]


class NegotiateResources_Response(NDRPacket):
    fields_desc = [NDRIntField("pdwcAccepted", 0), NDRIntField("status", 0)]


class SendReceive_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("phContext", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwcMessages", 0),
        NDRIntField("dwcbSizeOfBoxCar", None, size_of="rguchBoxCar"),
        NDRConfStrLenField("rguchBoxCar", "", size_is=lambda pkt: pkt.dwcbSizeOfBoxCar),
    ]


class SendReceive_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class TEARDOWN_TYPE(IntEnum):
    TT_FORCE = 0
    TT_PROBLEM = 2


class TearDownContext_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("contextHandle", NDRContextHandle(), NDRContextHandle),
        NDRInt3264EnumField("sRank", 0, SESSION_RANK),
        NDRInt3264EnumField("tearDownType", 0, TEARDOWN_TYPE),
    ]


class TearDownContext_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("contextHandle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class BeginTearDown_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("contextHandle", NDRContextHandle(), NDRContextHandle),
        NDRInt3264EnumField("tearDownType", 0, TEARDOWN_TYPE),
    ]


class BeginTearDown_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class PokeW_Request(NDRPacket):
    fields_desc = [
        NDRInt3264EnumField("sRank", 0, SESSION_RANK),
        NDRConfVarStrLenFieldUtf16(
            "pwszCalleeUuid", "", size_is=lambda pkt: pkt.GUID_LENGTH
        ),
        NDRConfVarStrLenFieldUtf16(
            "pwszHostName", "", size_is=lambda pkt: (pkt.MAX_COMPUTERNAME_LENGTH + 1)
        ),
        NDRConfVarStrLenFieldUtf16(
            "pwszUuidString", "", size_is=lambda pkt: pkt.GUID_LENGTH
        ),
        NDRIntField("dwcbSizeOfBlob", None, size_of="rguchBlob"),
        NDRConfStrLenField("rguchBlob", "", size_is=lambda pkt: pkt.dwcbSizeOfBlob),
    ]


class PokeW_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class BuildContextW_Request(NDRPacket):
    fields_desc = [
        NDRInt3264EnumField("sRank", 0, SESSION_RANK),
        NDRPacketField("BindVersionSet", BIND_VERSION_SET(), BIND_VERSION_SET),
        NDRConfVarStrLenFieldUtf16(
            "pwszCalleeUuid", "", size_is=lambda pkt: pkt.GUID_LENGTH
        ),
        NDRConfVarStrLenFieldUtf16(
            "pwszHostName", "", size_is=lambda pkt: (pkt.MAX_COMPUTERNAME_LENGTH + 1)
        ),
        NDRConfVarStrLenFieldUtf16(
            "pwszUuidString", "", size_is=lambda pkt: pkt.GUID_LENGTH
        ),
        NDRConfVarStrLenFieldUtf16(
            "pwszGuidIn", "", size_is=lambda pkt: pkt.GUID_LENGTH
        ),
        NDRConfVarStrLenFieldUtf16(
            "pwszGuidOut", "", size_is=lambda pkt: pkt.GUID_LENGTH
        ),
        NDRPacketField("pBoundVersionSet", BOUND_VERSION_SET(), BOUND_VERSION_SET),
        NDRIntField("dwcbSizeOfBlob", None, size_of="rguchBlob"),
        NDRConfStrLenField("rguchBlob", "", size_is=lambda pkt: pkt.dwcbSizeOfBlob),
    ]


class BuildContextW_Response(NDRPacket):
    fields_desc = [
        NDRConfVarStrLenFieldUtf16(
            "pwszGuidOut", "", size_is=lambda pkt: pkt.GUID_LENGTH
        ),
        NDRPacketField("pBoundVersionSet", BOUND_VERSION_SET(), BOUND_VERSION_SET),
        NDRPacketField("ppHandle", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


IXNREMOTE_OPNUMS = {
    0: DceRpcOp(Poke_Request, Poke_Response),
    1: DceRpcOp(BuildContext_Request, BuildContext_Response),
    2: DceRpcOp(NegotiateResources_Request, NegotiateResources_Response),
    3: DceRpcOp(SendReceive_Request, SendReceive_Response),
    4: DceRpcOp(TearDownContext_Request, TearDownContext_Response),
    5: DceRpcOp(BeginTearDown_Request, BeginTearDown_Response),
    6: DceRpcOp(PokeW_Request, PokeW_Response),
    7: DceRpcOp(BuildContextW_Request, BuildContextW_Response),
}
register_dcerpc_interface(
    name="IXnRemote",
    uuid=uuid.UUID("906B0CE0-C70B-1067-B317-00DD010662DA"),
    version="1.0",
    opnums=IXNREMOTE_OPNUMS,
)



### ms-coma.idl
"""
RPC definitions for the following interfaces:
- ICatalogSession (v0.0): 182C40FA-32E4-11D0-818B-00A0C9231C29
- ICatalog64BitSupport (v0.0): 1D118904-94B3-4A64-9FA6-ED432666A7B9
- ICatalogTableInfo (v0.0): A8927A41-D3CE-11D1-8472-006008B0E5CA
- ICatalogTableRead (v0.0): 0E3D6630-B46B-11D1-9D2D-006008B0E5CA
- ICatalogTableWrite (v0.0): 0E3D6631-B46B-11D1-9D2D-006008B0E5CA
- IRegister (v0.0): 8DB2180E-BD29-11D1-8B7E-00C04FD7A924
- IRegister2 (v0.0): 971668DC-C3FE-4EA1-9643-0C7230F494A1
- IImport (v0.0): C2BE6970-DF9E-11D1-8B87-00C04FD7A924
- IImport2 (v0.0): 1F7B1697-ECB2-4CBB-8A0E-75C427F4A6F0
- IExport (v0.0): CFADAC84-E12C-11D1-B34C-00C04F990D54
- IExport2 (v0.0): F131EA3E-B7BE-480E-A60D-51CB2785779E
- IAlternateLaunch (v0.0): 7F43B400-1A0E-4D57-BBC9-6B0C65F7A889
- ICatalogUtils (v0.0): 456129E2-1078-11D2-B0F9-00805FC73204
- ICatalogUtils2 (v0.0): C726744E-5735-4F08-8286-C510EE638FB6
- ICapabilitySupport (v0.0): 47CDE9A1-0BF6-11D2-8016-00C04FB9988E
- IContainerControl (v0.0): 3F3B1B86-DBBE-11D1-9DA6-00805F85CFE3
- IContainerControl2 (v0.0): 6C935649-30A6-4211-8687-C4C83E5FE1C7
- IReplicationUtil (v0.0): 98315903-7BE5-11D2-ADC1-00A02463D6E7
This file is auto-generated by midl-to-scapy, do not modify.
"""

from enum import IntEnum
import uuid

from scapy.fields import StrFixedLenField
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfFieldListField,
    NDRConfPacketListField,
    NDRConfStrLenField,
    NDRConfStrLenFieldUtf16,
    NDRConfVarStrLenField,
    NDRConfVarStrLenFieldUtf16,
    NDRConfVarStrNullField,
    NDRConfVarStrNullFieldUtf16,
    NDRFullPointerField,
    NDRIEEEFloatField,
    NDRInt3264EnumField,
    NDRIntField,
    NDRPacketField,
    NDRShortField,
    NDRSignedIntField,
    NDRSignedShortField,
    register_com_interface,
)


class InitializeSession_Request(NDRPacket):
    fields_desc = [
        NDRIEEEFloatField("flVerLower", 0),
        NDRIEEEFloatField("flVerUpper", 0),
        NDRSignedIntField("reserved", 0),
    ]


class InitializeSession_Response(NDRPacket):
    fields_desc = [NDRIEEEFloatField("pflVerSession", 0), NDRIntField("status", 0)]


class GetServerInformation_Request(NDRPacket):
    fields_desc = []


class GetServerInformation_Response(NDRPacket):
    fields_desc = [
        NDRSignedIntField("plReserved1", 0),
        NDRSignedIntField("plReserved2", 0),
        NDRSignedIntField("plReserved3", 0),
        NDRSignedIntField("plMultiplePartitionSupport", 0),
        NDRSignedIntField("plReserved4", 0),
        NDRSignedIntField("plReserved5", 0),
        NDRIntField("status", 0),
    ]


ICATALOGSESSION_OPNUMS = {  # 0: Opnum3NotUsedOnWire,
    # 1: Opnum4NotUsedOnWire,
    # 2: Opnum5NotUsedOnWire,
    # 3: Opnum6NotUsedOnWire,
    4: DceRpcOp(InitializeSession_Request, InitializeSession_Response),
    5: DceRpcOp(GetServerInformation_Request, GetServerInformation_Response),
}
register_com_interface(
    name="ICatalogSession",
    uuid=uuid.UUID("182C40FA-32E4-11D0-818B-00A0C9231C29"),
    opnums=ICATALOGSESSION_OPNUMS,
)


class SupportsMultipleBitness_Request(NDRPacket):
    fields_desc = []


class SupportsMultipleBitness_Response(NDRPacket):
    fields_desc = [
        NDRSignedIntField("pbSupportsMultipleBitness", 0),
        NDRIntField("status", 0),
    ]


class Initialize64BitQueryCellSupport_Request(NDRPacket):
    fields_desc = [NDRSignedIntField("bClientSupports64BitQueryCells", 0)]


class Initialize64BitQueryCellSupport_Response(NDRPacket):
    fields_desc = [
        NDRSignedIntField("pbServerSupports64BitQueryCells", 0),
        NDRIntField("status", 0),
    ]


ICATALOG64BITSUPPORT_OPNUMS = {
    0: DceRpcOp(SupportsMultipleBitness_Request, SupportsMultipleBitness_Response),
    1: DceRpcOp(
        Initialize64BitQueryCellSupport_Request,
        Initialize64BitQueryCellSupport_Response,
    ),
}
register_com_interface(
    name="ICatalog64BitSupport",
    uuid=uuid.UUID("1D118904-94B3-4A64-9FA6-ED432666A7B9"),
    opnums=ICATALOG64BITSUPPORT_OPNUMS,
)


class GUID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Data1", 0),
        NDRShortField("Data2", 0),
        NDRShortField("Data3", 0),
        StrFixedLenField("Data4", "", length=8),
    ]


class PropertyMeta(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("dataType", 0),
        NDRIntField("cbSize", 0),
        NDRIntField("flags", 0),
    ]


class GetClientTableInfo_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("pCatalogIdentifier", GUID(), GUID),
        NDRPacketField("pTableIdentifier", GUID(), GUID),
        NDRIntField("tableFlags", 0),
        NDRConfStrLenField(
            "pQueryCellArray", "", size_is=lambda pkt: pkt.cbQueryCellArray
        ),
        NDRIntField("cbQueryCellArray", None, size_of="pQueryCellArray"),
        NDRConfStrLenField(
            "pQueryComparison", "", size_is=lambda pkt: pkt.cbQueryComparison
        ),
        NDRIntField("cbQueryComparison", None, size_of="pQueryComparison"),
        NDRIntField("eQueryFormat", 0),
    ]


class GetClientTableInfo_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pRequiredFixedGuid", GUID(), GUID),
        NDRConfStrLenField("ppReserved1", "", size_is=lambda pkt: pkt.pcbReserved1),
        NDRIntField("pcbReserved1", None, size_of="ppReserved1"),
        NDRConfPacketListField(
            "ppAuxiliaryGuid",
            [],
            GUID,
            size_is=lambda pkt: pkt.pcAuxiliaryGuid,
            ptr_pack=True,
        ),
        NDRIntField("pcAuxiliaryGuid", None, size_of="ppAuxiliaryGuid"),
        NDRConfPacketListField(
            "ppPropertyMeta",
            [],
            PropertyMeta,
            size_is=lambda pkt: pkt.pcProperties,
            ptr_pack=True,
        ),
        NDRIntField("pcProperties", None, size_of="ppPropertyMeta"),
        NDRPacketField("piid", GUID(), GUID),
        StrFixedLenField("pItf", "", length=0),
        NDRConfStrLenField("ppReserved2", "", size_is=lambda pkt: pkt.pcbReserved2),
        NDRIntField("pcbReserved2", None, size_of="ppReserved2"),
        NDRIntField("status", 0),
    ]


ICATALOGTABLEINFO_OPNUMS = {
    0: DceRpcOp(GetClientTableInfo_Request, GetClientTableInfo_Response)
}
register_com_interface(
    name="ICatalogTableInfo",
    uuid=uuid.UUID("A8927A41-D3CE-11D1-8472-006008B0E5CA"),
    opnums=ICATALOGTABLEINFO_OPNUMS,
)


class ReadTable_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("pCatalogIdentifier", GUID(), GUID),
        NDRPacketField("pTableIdentifier", GUID(), GUID),
        NDRIntField("tableFlags", 0),
        NDRConfStrLenField(
            "pQueryCellArray", "", size_is=lambda pkt: pkt.cbQueryCellArray
        ),
        NDRIntField("cbQueryCellArray", None, size_of="pQueryCellArray"),
        NDRConfStrLenField(
            "pQueryComparison", "", size_is=lambda pkt: pkt.cbQueryComparison
        ),
        NDRIntField("cbQueryComparison", None, size_of="pQueryComparison"),
        NDRIntField("eQueryFormat", 0),
    ]


class ReadTable_Response(NDRPacket):
    fields_desc = [
        NDRConfStrLenField(
            "ppTableDataFixed", "", size_is=lambda pkt: pkt.pcbTableDataFixed
        ),
        NDRIntField("pcbTableDataFixed", None, size_of="ppTableDataFixed"),
        NDRConfStrLenField(
            "ppTableDataVariable", "", size_is=lambda pkt: pkt.pcbTableDataVariable
        ),
        NDRIntField("pcbTableDataVariable", None, size_of="ppTableDataVariable"),
        NDRConfStrLenField(
            "ppTableDetailedErrors", "", size_is=lambda pkt: pkt.pcbTableDetailedErrors
        ),
        NDRIntField("pcbTableDetailedErrors", None, size_of="ppTableDetailedErrors"),
        NDRConfStrLenField("ppReserved1", "", size_is=lambda pkt: pkt.pcbReserved1),
        NDRIntField("pcbReserved1", None, size_of="ppReserved1"),
        NDRConfStrLenField("ppReserved2", "", size_is=lambda pkt: pkt.pcbReserved2),
        NDRIntField("pcbReserved2", None, size_of="ppReserved2"),
        NDRIntField("status", 0),
    ]


ICATALOGTABLEREAD_OPNUMS = {0: DceRpcOp(ReadTable_Request, ReadTable_Response)}
register_com_interface(
    name="ICatalogTableRead",
    uuid=uuid.UUID("0E3D6630-B46B-11D1-9D2D-006008B0E5CA"),
    opnums=ICATALOGTABLEREAD_OPNUMS,
)


class WriteTable_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("pCatalogIdentifier", GUID(), GUID),
        NDRPacketField("pTableIdentifier", GUID(), GUID),
        NDRIntField("tableFlags", 0),
        NDRConfStrLenField(
            "pQueryCellArray", "", size_is=lambda pkt: pkt.cbQueryCellArray
        ),
        NDRIntField("cbQueryCellArray", None, size_of="pQueryCellArray"),
        NDRConfStrLenField(
            "pQueryComparison", "", size_is=lambda pkt: pkt.cbQueryComparison
        ),
        NDRIntField("cbQueryComparison", None, size_of="pQueryComparison"),
        NDRIntField("eQueryFormat", 0),
        NDRConfStrLenField(
            "pTableDataFixedWrite", "", size_is=lambda pkt: pkt.cbTableDataFixedWrite
        ),
        NDRIntField("cbTableDataFixedWrite", None, size_of="pTableDataFixedWrite"),
        NDRConfStrLenField(
            "pTableDataVariable", "", size_is=lambda pkt: pkt.cbTableDataVariable
        ),
        NDRIntField("cbTableDataVariable", None, size_of="pTableDataVariable"),
        NDRConfStrLenField("pReserved1", "", size_is=lambda pkt: pkt.cbReserved1),
        NDRIntField("cbReserved1", None, size_of="pReserved1"),
        NDRConfStrLenField("pReserved2", "", size_is=lambda pkt: pkt.cbReserved2),
        NDRIntField("cbReserved2", None, size_of="pReserved2"),
        NDRConfStrLenField("pReserved3", "", size_is=lambda pkt: pkt.cbReserved3),
        NDRIntField("cbReserved3", None, size_of="pReserved3"),
    ]


class WriteTable_Response(NDRPacket):
    fields_desc = [
        NDRConfStrLenField(
            "ppTableDetailedErrors", "", size_is=lambda pkt: pkt.pcbTableDetailedErrors
        ),
        NDRIntField("pcbTableDetailedErrors", None, size_of="ppTableDetailedErrors"),
        NDRIntField("status", 0),
    ]


ICATALOGTABLEWRITE_OPNUMS = {0: DceRpcOp(WriteTable_Request, WriteTable_Response)}
register_com_interface(
    name="ICatalogTableWrite",
    uuid=uuid.UUID("0E3D6631-B46B-11D1-9D2D-006008B0E5CA"),
    opnums=ICATALOGTABLEWRITE_OPNUMS,
)


class RegisterModule_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("ConglomerationIdentifier", GUID(), GUID),
        NDRConfVarStrLenFieldUtf16("ppModules", "", size_is=lambda pkt: pkt.cModules),
        NDRIntField("cModules", None, size_of="ppModules"),
        NDRIntField("dwFlags", 0),
        NDRConfPacketListField(
            "pRequestedCLSIDs", [], GUID, size_is=lambda pkt: pkt.cRequested
        ),
        NDRIntField("cRequested", None, size_of="pRequestedCLSIDs"),
    ]


class RegisterModule_Response(NDRPacket):
    fields_desc = [
        NDRConfFieldListField(
            "ppModuleFlags",
            [],
            NDRIntField("", 0),
            size_is=lambda pkt: pkt.cModules,
            ptr_pack=True,
        ),
        NDRIntField("pcResults", None, size_of="ppResultHRs"),
        NDRConfPacketListField(
            "ppResultCLSIDs", [], GUID, size_is=lambda pkt: pkt.pcResults, ptr_pack=True
        ),
        NDRConfVarStrLenFieldUtf16(
            "ppResultNames", "", size_is=lambda pkt: pkt.pcResults
        ),
        NDRConfFieldListField(
            "ppResultFlags",
            [],
            NDRIntField("", 0),
            size_is=lambda pkt: pkt.pcResults,
            ptr_pack=True,
        ),
        NDRConfFieldListField(
            "ppResultHRs",
            [],
            NDRSignedIntField("", 0),
            size_is=lambda pkt: pkt.pcResults,
            ptr_pack=True,
        ),
        NDRIntField("status", 0),
    ]


IREGISTER_OPNUMS = {
    0: DceRpcOp(RegisterModule_Request, RegisterModule_Response),
    # 1: Opnum4NotUsedOnWire
}
register_com_interface(
    name="IRegister",
    uuid=uuid.UUID("8DB2180E-BD29-11D1-8B7E-00C04FD7A924"),
    opnums=IREGISTER_OPNUMS,
)


class eComponentType(IntEnum):
    eCT_UNKNOWN = 0
    eCT_32BIT = 1
    eCT_64BIT = 2
    eCT_NATIVE = 4096


class CreateFullConfiguration_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("pwszConglomerationIdOrName", ""),
        NDRConfVarStrNullFieldUtf16("pwszCLSIDOrProgId", ""),
        NDRInt3264EnumField("ctComponentType", 0, eComponentType),
    ]


class CreateFullConfiguration_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class CreateLegacyConfiguration_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("pwszConglomerationIdOrName", ""),
        NDRConfVarStrNullFieldUtf16("pwszCLSIDOrProgId", ""),
        NDRInt3264EnumField("ctComponentType", 0, eComponentType),
    ]


class CreateLegacyConfiguration_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class PromoteLegacyConfiguration_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("pwszConglomerationIdOrName", ""),
        NDRConfVarStrNullFieldUtf16("pwszCLSIDOrProgId", ""),
        NDRInt3264EnumField("ctComponentType", 0, eComponentType),
    ]


class PromoteLegacyConfiguration_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class RegisterModule2_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("ConglomerationIdentifier", GUID(), GUID),
        NDRPacketField("PartitionIdentifier", GUID(), GUID),
        NDRConfVarStrLenFieldUtf16("ppModules", "", size_is=lambda pkt: pkt.cModules),
        NDRIntField("cModules", None, size_of="ppModules"),
        NDRIntField("dwFlags", 0),
        NDRConfPacketListField(
            "pRequestedCLSIDs", [], GUID, size_is=lambda pkt: pkt.cRequested
        ),
        NDRIntField("cRequested", None, size_of="pRequestedCLSIDs"),
    ]


class RegisterModule2_Response(NDRPacket):
    fields_desc = [
        NDRConfFieldListField(
            "ppModuleFlags",
            [],
            NDRIntField("", 0),
            size_is=lambda pkt: pkt.cModules,
            ptr_pack=True,
        ),
        NDRIntField("pcResults", None, size_of="ppResultHRs"),
        NDRConfPacketListField(
            "ppResultCLSIDs", [], GUID, size_is=lambda pkt: pkt.pcResults, ptr_pack=True
        ),
        NDRConfVarStrLenFieldUtf16(
            "ppResultNames", "", size_is=lambda pkt: pkt.pcResults
        ),
        NDRConfFieldListField(
            "ppResultFlags",
            [],
            NDRIntField("", 0),
            size_is=lambda pkt: pkt.pcResults,
            ptr_pack=True,
        ),
        NDRConfFieldListField(
            "ppResultHRs",
            [],
            NDRSignedIntField("", 0),
            size_is=lambda pkt: pkt.pcResults,
            ptr_pack=True,
        ),
        NDRIntField("status", 0),
    ]


IREGISTER2_OPNUMS = {
    0: DceRpcOp(CreateFullConfiguration_Request, CreateFullConfiguration_Response),
    1: DceRpcOp(CreateLegacyConfiguration_Request, CreateLegacyConfiguration_Response),
    2: DceRpcOp(
        PromoteLegacyConfiguration_Request, PromoteLegacyConfiguration_Response
    ),
    # 3: Opnum6NotUsedOnWire,
    # 4: Opnum7NotUsedOnWire,
    5: DceRpcOp(RegisterModule2_Request, RegisterModule2_Response),
    # 6: Opnum9NotUsedOnWire
}
register_com_interface(
    name="IRegister2",
    uuid=uuid.UUID("971668DC-C3FE-4EA1-9643-0C7230F494A1"),
    opnums=IREGISTER2_OPNUMS,
)


class ImportFromFile_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszModuleDestination", "")),
        NDRConfVarStrNullFieldUtf16("pwszInstallerPackage", ""),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszUser", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszPassword", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszRemoteServerName", "")),
        NDRIntField("dwFlags", 0),
        NDRPacketField("reserved1", GUID(), GUID),
        NDRIntField("reserved2", 0),
    ]


class ImportFromFile_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pcModules", None, size_of="ppModules"),
        NDRConfFieldListField(
            "ppModuleFlags",
            [],
            NDRIntField("", 0),
            size_is=lambda pkt: pkt.pcModules,
            ptr_pack=True,
        ),
        NDRConfVarStrLenFieldUtf16("ppModules", "", size_is=lambda pkt: pkt.pcModules),
        NDRIntField("pcComponents", None, size_of="ppResultHRs"),
        NDRConfPacketListField(
            "ppResultCLSIDs",
            [],
            GUID,
            size_is=lambda pkt: pkt.pcComponents,
            ptr_pack=True,
        ),
        NDRConfVarStrLenFieldUtf16(
            "ppResultNames", "", size_is=lambda pkt: pkt.pcComponents
        ),
        NDRConfFieldListField(
            "ppResultFlags",
            [],
            NDRIntField("", 0),
            size_is=lambda pkt: pkt.pcComponents,
            ptr_pack=True,
        ),
        NDRConfFieldListField(
            "ppResultHRs",
            [],
            NDRSignedIntField("", 0),
            size_is=lambda pkt: pkt.pcComponents,
            ptr_pack=True,
        ),
        NDRIntField("status", 0),
    ]


class QueryFile_Request(NDRPacket):
    fields_desc = [NDRConfVarStrNullFieldUtf16("pwszInstallerPackage", "")]


class QueryFile_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pdwConglomerations", None, size_of="ppDescriptions"),
        NDRConfVarStrLenFieldUtf16(
            "ppNames", "", size_is=lambda pkt: pkt.pdwConglomerations
        ),
        NDRConfVarStrLenFieldUtf16(
            "ppDescriptions", "", size_is=lambda pkt: pkt.pdwConglomerations
        ),
        NDRIntField("pdwUsers", 0),
        NDRIntField("pdwIsProxy", 0),
        NDRIntField("pcModules", None, size_of="ppModules"),
        NDRConfVarStrLenFieldUtf16("ppModules", "", size_is=lambda pkt: pkt.pcModules),
        NDRIntField("status", 0),
    ]


IIMPORT_OPNUMS = {
    0: DceRpcOp(ImportFromFile_Request, ImportFromFile_Response),
    1: DceRpcOp(QueryFile_Request, QueryFile_Response),
    # 2: Opnum5NotUsedOnWire,
    # 3: Opnum6NotUsedOnWire
}
register_com_interface(
    name="IImport",
    uuid=uuid.UUID("C2BE6970-DF9E-11D1-8B87-00C04FD7A924"),
    opnums=IIMPORT_OPNUMS,
)


class SetPartition_Request(NDRPacket):
    fields_desc = [NDRPacketField("pPartitionIdentifier", GUID(), GUID)]


class SetPartition_Response(NDRPacket):
    fields_desc = [NDRPacketField("pReserved", GUID(), GUID), NDRIntField("status", 0)]


IIMPORT2_OPNUMS = {
    0: DceRpcOp(SetPartition_Request, SetPartition_Response),
    # 1: Opnum4NotUsedOnWire,
    # 2: Opnum5NotUsedOnWire
}
register_com_interface(
    name="IImport2",
    uuid=uuid.UUID("1F7B1697-ECB2-4CBB-8A0E-75C427F4A6F0"),
    opnums=IIMPORT2_OPNUMS,
)


class ExportConglomeration_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("pConglomerationIdentifier", GUID(), GUID),
        NDRShortField("pwszInstallerPackage", 0),
        NDRShortField("pwszReserved", 0),
        NDRIntField("dwFlags", 0),
    ]


class ExportConglomeration_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


IEXPORT_OPNUMS = {
    0: DceRpcOp(ExportConglomeration_Request, ExportConglomeration_Response),
    # 1: Opnum4NotUsedOnWire,
    # 2: Opnum5NotUsedOnWire,
    # 3: Opnum6NotUsedOnWire
}
register_com_interface(
    name="IExport",
    uuid=uuid.UUID("CFADAC84-E12C-11D1-B34C-00C04F990D54"),
    opnums=IEXPORT_OPNUMS,
)


class ExportPartition_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("pPartitionIdentifier", GUID(), GUID),
        NDRShortField("pwszInstallerPackage", 0),
        NDRShortField("pwszReserved", 0),
        NDRIntField("dwFlags", 0),
    ]


class ExportPartition_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


IEXPORT2_OPNUMS = {0: DceRpcOp(ExportPartition_Request, ExportPartition_Response)}
register_com_interface(
    name="IExport2",
    uuid=uuid.UUID("F131EA3E-B7BE-480E-A60D-51CB2785779E"),
    opnums=IEXPORT2_OPNUMS,
)


class FLAGGED_WORD_BLOB(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["asData"]
    fields_desc = [
        NDRIntField("cBytes", 0),
        NDRIntField("clSize", None, size_of="asData"),
        NDRConfStrLenFieldUtf16(
            "asData", "", size_is=lambda pkt: pkt.clSize, conformant_in_struct=True
        ),
    ]


class CreateConfiguration_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("ConglomerationIdentifier", GUID(), GUID),
        NDRPacketField("bstrConfigurationName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRIntField("dwStartType", 0),
        NDRIntField("dwErrorControl", 0),
        NDRPacketField("bstrDependencies", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("bstrRunAs", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("bstrPassword", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRSignedShortField("bDesktopOk", 0),
    ]


class CreateConfiguration_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class DeleteConfiguration_Request(NDRPacket):
    fields_desc = [NDRPacketField("ConglomerationIdentifier", GUID(), GUID)]


class DeleteConfiguration_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


IALTERNATELAUNCH_OPNUMS = {
    0: DceRpcOp(CreateConfiguration_Request, CreateConfiguration_Response),
    1: DceRpcOp(DeleteConfiguration_Request, DeleteConfiguration_Response),
}
register_com_interface(
    name="IAlternateLaunch",
    uuid=uuid.UUID("7F43B400-1A0E-4D57-BBC9-6B0C65F7A889"),
    opnums=IALTERNATELAUNCH_OPNUMS,
)


class ValidateUser_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszPrincipalName", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszPassword", "")),
    ]


class ValidateUser_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class WaitForEndWrites_Request(NDRPacket):
    fields_desc = []


class WaitForEndWrites_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class GetEventClassesForIID_Request(NDRPacket):
    fields_desc = [NDRFullPointerField(NDRConfVarStrNullFieldUtf16("wszIID", ""))]


class GetEventClassesForIID_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pcClasses", None, size_of="pawszDescriptions"),
        NDRConfVarStrLenFieldUtf16(
            "pawszCLSIDs", "", size_is=lambda pkt: pkt.pcClasses
        ),
        NDRConfVarStrLenFieldUtf16(
            "pawszProgIDs", "", size_is=lambda pkt: pkt.pcClasses
        ),
        NDRConfVarStrLenFieldUtf16(
            "pawszDescriptions", "", size_is=lambda pkt: pkt.pcClasses
        ),
        NDRIntField("status", 0),
    ]


ICATALOGUTILS_OPNUMS = {
    0: DceRpcOp(ValidateUser_Request, ValidateUser_Response),
    1: DceRpcOp(WaitForEndWrites_Request, WaitForEndWrites_Response),
    2: DceRpcOp(GetEventClassesForIID_Request, GetEventClassesForIID_Response),
}
register_com_interface(
    name="ICatalogUtils",
    uuid=uuid.UUID("456129E2-1078-11D2-B0F9-00805FC73204"),
    opnums=ICATALOGUTILS_OPNUMS,
)


class CopyConglomerations_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("pwszSourcePartition", ""),
        NDRConfVarStrNullFieldUtf16("pwszDestPartition", ""),
        NDRIntField("cConglomerations", None, size_of="ppwszConglomerationNamesOrIds"),
        NDRConfVarStrLenFieldUtf16(
            "ppwszConglomerationNamesOrIds",
            "",
            size_is=lambda pkt: pkt.cConglomerations,
        ),
    ]


class CopyConglomerations_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class CopyComponentConfiguration_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("pwszSourceConglomeration", ""),
        NDRConfVarStrNullFieldUtf16("pwszComponent", ""),
        NDRConfVarStrNullFieldUtf16("pwszDestConglomeration", ""),
    ]


class CopyComponentConfiguration_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class AliasComponent_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("pwszSourceConglomeration", ""),
        NDRConfVarStrNullFieldUtf16("pwszComponent", ""),
        NDRConfVarStrNullFieldUtf16("pwszDestConglomeration", ""),
        NDRPacketField("pNewCLSID", GUID(), GUID),
        NDRConfVarStrNullFieldUtf16("pwszNewProgID", ""),
    ]


class AliasComponent_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class MoveComponentConfiguration_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("pwszSourceConglomeration", ""),
        NDRConfVarStrNullFieldUtf16("pwszComponent", ""),
        NDRConfVarStrNullFieldUtf16("pwszDestinationConglomeration", ""),
    ]


class MoveComponentConfiguration_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class GetEventClassesForIID2_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("wszIID", "")),
        NDRPacketField("PartitionId", GUID(), GUID),
    ]


class GetEventClassesForIID2_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pcClasses", None, size_of="padwIsPrivate"),
        NDRConfVarStrLenFieldUtf16(
            "pawszCLSIDs", "", size_is=lambda pkt: pkt.pcClasses
        ),
        NDRConfVarStrLenFieldUtf16(
            "pawszProgIDs", "", size_is=lambda pkt: pkt.pcClasses
        ),
        NDRConfVarStrLenFieldUtf16(
            "pawszDescriptions", "", size_is=lambda pkt: pkt.pcClasses
        ),
        NDRConfVarStrLenFieldUtf16(
            "pawszConglomerationIDs", "", size_is=lambda pkt: pkt.pcClasses
        ),
        NDRConfFieldListField(
            "padwIsPrivate",
            [],
            NDRIntField("", 0),
            size_is=lambda pkt: pkt.pcClasses,
            ptr_pack=True,
        ),
        NDRIntField("status", 0),
    ]


class IsSafeToDelete_Request(NDRPacket):
    fields_desc = [NDRPacketField("bstrFile", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)]


class IsSafeToDelete_Response(NDRPacket):
    fields_desc = [NDRSignedIntField("pInUse", 0), NDRIntField("status", 0)]


class FlushPartitionCache_Request(NDRPacket):
    fields_desc = []


class FlushPartitionCache_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class SRPLevelInfo(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("dwSRPLevel", 0),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("wszFriendlyName", ""), deferred=True
        ),
    ]


class EnumerateSRPLevels_Request(NDRPacket):
    fields_desc = [NDRIntField("Locale", 0)]


class EnumerateSRPLevels_Response(NDRPacket):
    fields_desc = [
        NDRSignedIntField("cLevels", None, size_of="aSRPLevels"),
        NDRConfPacketListField(
            "aSRPLevels",
            [],
            SRPLevelInfo,
            size_is=lambda pkt: pkt.cLevels,
            ptr_pack=True,
        ),
        NDRIntField("status", 0),
    ]


class GetComponentVersions_Request(NDRPacket):
    fields_desc = [NDRShortField("pwszClsidOrProgId", 0)]


class GetComponentVersions_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pdwVersions", None, size_of="ppBitness"),
        NDRConfPacketListField(
            "ppPartitionIDs",
            [],
            GUID,
            size_is=lambda pkt: pkt.pdwVersions,
            ptr_pack=True,
        ),
        NDRConfPacketListField(
            "ppConglomerationIDs",
            [],
            GUID,
            size_is=lambda pkt: pkt.pdwVersions,
            ptr_pack=True,
        ),
        NDRConfFieldListField(
            "ppIsPrivate",
            [],
            NDRSignedIntField("", 0),
            size_is=lambda pkt: pkt.pdwVersions,
            ptr_pack=True,
        ),
        NDRConfFieldListField(
            "ppBitness",
            [],
            NDRSignedIntField("", 0),
            size_is=lambda pkt: pkt.pdwVersions,
            ptr_pack=True,
        ),
        NDRIntField("status", 0),
    ]


ICATALOGUTILS2_OPNUMS = {
    0: DceRpcOp(CopyConglomerations_Request, CopyConglomerations_Response),
    1: DceRpcOp(
        CopyComponentConfiguration_Request, CopyComponentConfiguration_Response
    ),
    2: DceRpcOp(AliasComponent_Request, AliasComponent_Response),
    3: DceRpcOp(
        MoveComponentConfiguration_Request, MoveComponentConfiguration_Response
    ),
    4: DceRpcOp(GetEventClassesForIID2_Request, GetEventClassesForIID2_Response),
    5: DceRpcOp(IsSafeToDelete_Request, IsSafeToDelete_Response),
    6: DceRpcOp(FlushPartitionCache_Request, FlushPartitionCache_Response),
    7: DceRpcOp(EnumerateSRPLevels_Request, EnumerateSRPLevels_Response),
    8: DceRpcOp(GetComponentVersions_Request, GetComponentVersions_Response),
}
register_com_interface(
    name="ICatalogUtils2",
    uuid=uuid.UUID("C726744E-5735-4F08-8286-C510EE638FB6"),
    opnums=ICATALOGUTILS2_OPNUMS,
)


class CatSrvServices(IntEnum):
    css_lb = 1


class Start_Request(NDRPacket):
    fields_desc = [NDRInt3264EnumField("i_css", 0, CatSrvServices)]


class Start_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class Stop_Request(NDRPacket):
    fields_desc = [NDRInt3264EnumField("i_css", 0, CatSrvServices)]


class Stop_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class IsInstalled_Request(NDRPacket):
    fields_desc = [NDRInt3264EnumField("i_css", 0, CatSrvServices)]


class IsInstalled_Response(NDRPacket):
    fields_desc = [NDRIntField("pulStatus", 0), NDRIntField("status", 0)]


class CatSrvServiceState(IntEnum):
    css_serviceStopped = 0
    css_serviceStartPending = 1
    css_serviceStopPending = 2
    css_serviceRunning = 3
    css_serviceContinuePending = 4
    css_servicePausePending = 5
    css_servicePaused = 6
    css_serviceUnknownState = 7


class IsRunning_Request(NDRPacket):
    fields_desc = [NDRInt3264EnumField("i_css", 0, CatSrvServices)]


class IsRunning_Response(NDRPacket):
    fields_desc = [
        NDRInt3264EnumField("pulStates", 0, CatSrvServiceState),
        NDRIntField("status", 0),
    ]


ICAPABILITYSUPPORT_OPNUMS = {
    0: DceRpcOp(Start_Request, Start_Response),
    1: DceRpcOp(Stop_Request, Stop_Response),
    # 2: Opnum5NotUsedOnWire,
    # 3: Opnum6NotUsedOnWire,
    4: DceRpcOp(IsInstalled_Request, IsInstalled_Response),
    5: DceRpcOp(IsRunning_Request, IsRunning_Response),
    # 6: Opnum9NotUsedOnWire
}
register_com_interface(
    name="ICapabilitySupport",
    uuid=uuid.UUID("47CDE9A1-0BF6-11D2-8016-00C04FB9988E"),
    opnums=ICAPABILITYSUPPORT_OPNUMS,
)


class CreateContainer_Request(NDRPacket):
    fields_desc = [NDRPacketField("pConglomerationIdentifier", GUID(), GUID)]


class CreateContainer_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ShutdownContainers_Request(NDRPacket):
    fields_desc = [NDRPacketField("pConglomerationIdentifier", GUID(), GUID)]


class ShutdownContainers_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class RefreshComponents_Request(NDRPacket):
    fields_desc = []


class RefreshComponents_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


ICONTAINERCONTROL_OPNUMS = {
    0: DceRpcOp(CreateContainer_Request, CreateContainer_Response),
    1: DceRpcOp(ShutdownContainers_Request, ShutdownContainers_Response),
    2: DceRpcOp(RefreshComponents_Request, RefreshComponents_Response),
}
register_com_interface(
    name="IContainerControl",
    uuid=uuid.UUID("3F3B1B86-DBBE-11D1-9DA6-00805F85CFE3"),
    opnums=ICONTAINERCONTROL_OPNUMS,
)


class ShutdownContainer_Request(NDRPacket):
    fields_desc = [NDRPacketField("ContainerIdentifier", GUID(), GUID)]


class ShutdownContainer_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class PauseContainer_Request(NDRPacket):
    fields_desc = [NDRPacketField("ContainerIdentifier", GUID(), GUID)]


class PauseContainer_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ResumeContainer_Request(NDRPacket):
    fields_desc = [NDRPacketField("ContainerIdentifier", GUID(), GUID)]


class ResumeContainer_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class IsContainerPaused_Request(NDRPacket):
    fields_desc = [NDRPacketField("ContainerIdentifier", GUID(), GUID)]


class IsContainerPaused_Response(NDRPacket):
    fields_desc = [NDRSignedIntField("bPaused", 0), NDRIntField("status", 0)]


class InstanceContainer(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRPacketField("ConglomerationID", GUID(), GUID),
        NDRPacketField("PartitionID", GUID(), GUID),
        NDRPacketField("ContainerID", GUID(), GUID),
        NDRIntField("dwProcessID", 0),
        NDRSignedIntField("bPaused", 0),
        NDRSignedIntField("bRecycled", 0),
    ]


class GetRunningContainers_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("PartitionId", GUID(), GUID),
        NDRPacketField("ConglomerationId", GUID(), GUID),
    ]


class GetRunningContainers_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pdwNumContainers", None, size_of="ppContainers"),
        NDRConfPacketListField(
            "ppContainers",
            [],
            InstanceContainer,
            size_is=lambda pkt: pkt.pdwNumContainers,
            ptr_pack=True,
        ),
        NDRIntField("status", 0),
    ]


class GetContainerIDFromProcessID_Request(NDRPacket):
    fields_desc = [NDRIntField("dwPID", 0)]


class GetContainerIDFromProcessID_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrContainerID", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class RecycleContainer_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("ContainerIdentifier", GUID(), GUID),
        NDRSignedIntField("lReasonCode", 0),
    ]


class RecycleContainer_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class GetContainerIDFromConglomerationID_Request(NDRPacket):
    fields_desc = [NDRPacketField("ConglomerationIdentifier", GUID(), GUID)]


class GetContainerIDFromConglomerationID_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("ContainerIdentifier", GUID(), GUID),
        NDRIntField("status", 0),
    ]


ICONTAINERCONTROL2_OPNUMS = {
    0: DceRpcOp(ShutdownContainer_Request, ShutdownContainer_Response),
    1: DceRpcOp(PauseContainer_Request, PauseContainer_Response),
    2: DceRpcOp(ResumeContainer_Request, ResumeContainer_Response),
    3: DceRpcOp(IsContainerPaused_Request, IsContainerPaused_Response),
    4: DceRpcOp(GetRunningContainers_Request, GetRunningContainers_Response),
    5: DceRpcOp(
        GetContainerIDFromProcessID_Request, GetContainerIDFromProcessID_Response
    ),
    6: DceRpcOp(RecycleContainer_Request, RecycleContainer_Response),
    7: DceRpcOp(
        GetContainerIDFromConglomerationID_Request,
        GetContainerIDFromConglomerationID_Response,
    ),
}
register_com_interface(
    name="IContainerControl2",
    uuid=uuid.UUID("6C935649-30A6-4211-8687-C4C83E5FE1C7"),
    opnums=ICONTAINERCONTROL2_OPNUMS,
)


class CreateShare_Request(NDRPacket):
    fields_desc = [NDRShortField("pwszShareName", 0), NDRShortField("pwszPath", 0)]


class CreateShare_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class CreateEmptyDir_Request(NDRPacket):
    fields_desc = [NDRShortField("pwszPath", 0)]


class CreateEmptyDir_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class RemoveShare_Request(NDRPacket):
    fields_desc = [NDRShortField("pwszShareName", 0)]


class RemoveShare_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class BeginReplicationAsTarget_Request(NDRPacket):
    fields_desc = [NDRShortField("pwszBaseReplicationDir", 0)]


class BeginReplicationAsTarget_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class QueryConglomerationPassword_Request(NDRPacket):
    fields_desc = [NDRPacketField("ConglomerationId", GUID(), GUID)]


class QueryConglomerationPassword_Response(NDRPacket):
    fields_desc = [
        NDRConfStrLenField("ppvPassword", "", size_is=lambda pkt: pkt.pcbPassword),
        NDRIntField("pcbPassword", None, size_of="ppvPassword"),
        NDRIntField("status", 0),
    ]


class CreateReplicationDir_Request(NDRPacket):
    fields_desc = []


class CreateReplicationDir_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRShortField("ppwszBaseReplicationDir", 0)),
        NDRIntField("status", 0),
    ]


IREPLICATIONUTIL_OPNUMS = {
    0: DceRpcOp(CreateShare_Request, CreateShare_Response),
    1: DceRpcOp(CreateEmptyDir_Request, CreateEmptyDir_Response),
    2: DceRpcOp(RemoveShare_Request, RemoveShare_Response),
    3: DceRpcOp(BeginReplicationAsTarget_Request, BeginReplicationAsTarget_Response),
    4: DceRpcOp(
        QueryConglomerationPassword_Request, QueryConglomerationPassword_Response
    ),
    5: DceRpcOp(CreateReplicationDir_Request, CreateReplicationDir_Response),
}
register_com_interface(
    name="IReplicationUtil",
    uuid=uuid.UUID("98315903-7BE5-11D2-ADC1-00A02463D6E7"),
    opnums=IREPLICATIONUTIL_OPNUMS,
)
